import datetime
import json

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.views.decorators.csrf import csrf_exempt

from errata.models import Bundle, Distro, Erratum, Package, Release

from security.decorators import token_auth

def erratum(request, launchpad):
    erratum = get_object_or_404(Erratum, launchpad=launchpad, disclosed=True)

    return render_to_response('errata/erratum.html', {
        'erratum': erratum})

def index(request):
    errata = Erratum.objects.filter(disclosed=True).order_by('-created')[:30]

    return render_to_response('errata/index.html', {
        'errata': errata,
        'recent_active': True})

def search(request):
    query = request.GET.get('query').strip()

    errata = Erratum.objects.filter(
        Q(description__icontains=query) |
        Q(title__icontains=query) |
        Q(cve__icontains=query) |
        Q(ossa__icontains=query) |
        Q(launchpad=query) |
        Q(fixed_package__git_project=query) |
        Q(fixed_package__git_commit=query),
        disclosed=True)

    return render_to_response('errata/search.html', {
        'errata': errata,
        'results': len(errata),
        'query': query,
    })

@token_auth
def api_errata_recent(request):
    ctimereq = request.GET.get('ctime', None)
    startreq = request.GET.get('start')
    endreq = request.GET.get('end')

    if(ctimereq != None):
        errataqs = Erratum.objects.filter(
            created__gt=(datetime.datetime.now()-datetime.timedelta(seconds=int(ctimereq))))
    elif(startreq != None and endreq != None):
        errataqs = Erratum.objects.filter(
            created__gt=datetime.datetime.fromtimestamp(int(startreq)),
            created__lt=datetime.datetime.fromtimestamp(int(endreq)))
    else:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'reason': 'ctime or both of start and end GET parameters must be specified',
            }),
            content_type='application/json',
        )

    errata = []
    for erratum in errataqs:
        errata.append({
            'launchpad': erratum.launchpad,
            'title': erratum.title,
            'url': 'http://%s/erratum/%s.html' % (request.META['HTTP_HOST'], erratum.launchpad),
            'cve': erratum.cve,
            'created': str(erratum.created),
        })


    return HttpResponse(
        content=json.dumps(errata),
        content_type='application/json',
    )

@token_auth
def api_erratum(request, launchpad):
    try:
        erratum = Erratum.objects.get(launchpad=launchpad)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'launchpad': launchpad,
                'reason': 'Erratum does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    return HttpResponse(
        content=json.dumps({
            'launchpad': erratum.launchpad,
            'title': erratum.title,
            'description': erratum.description,
            'cve': erratum.cve,
        }), content_type='application/json'
    )

@token_auth
def api_erratum_update(request):
    launchpad = request.POST.get('launchpad')
    disclosed = request.POST.get('disclosed')

    if disclosed == 'true':
        disclosed = True
    else:
        disclosed = False

    try:
        e = Erratum.objects.get(launchpad=int(launchpad))
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'launchpad': launchpad,
                'reason': 'Erratum does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    e.disclosed = disclosed
    e.save()

    return HttpResponse(
        content=json.dumps({
            'status': 'ok',
            'launchpad': launchpad,
            'disclosed': disclosed,
        }),
        content_type='application/json',
    )

@token_auth
def api_package(request):
    req_release = request.GET.get('release')
    req_distro = request.GET.get('distro')
    req_package = request.GET.get('package')
    req_version = request.GET.get('version')

    try:
        r = Release.objects.get(name=req_release)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'release': req_release,
                'reason': 'Release does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    try:
        d = Distro.objects.get(name=req_distro, release=r)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'distro': req_distro,
                'release': r.name,
                'reason': 'Distro does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    try:
        package = Package.objects.get(
            name=req_package, version=req_version, release=r, distro=d)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'distro': d.name,
                'release': r.name,
                'package': req_package,
                'version': req_version,
                'reason': 'Package does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    return HttpResponse(
        content=json.dumps({
            'status': 'ok',
            'package': {
                'name': package.name,
                'version': package.version,
                'bundle': package.bundle.bundleid,
                'distro': package.distro.name,
                'release': package.release.name,
                'launchpad_affected': [x.launchpad for x in package.errata_affected.all()],
                'launchpad_fixed': [x.launchpad for x in package.errata_fixed.all()],
                'git_project': package.git_project,
                'git_commit': package.git_commit,
            }
        })
    )

@token_auth
def api_package_list(request):
    packages = Package.objects.all()

    package_out = []
    for package in packages:
        bundle = None
        if package.bundle:
            bundle = package.bundle.bundleid

        package_out.append({
            'name': package.name,
            'version': package.version,
            'bundle': bundle,
            'distro': package.distro.name,
            'release': package.release.name,
            'launchpad_affected': [x.launchpad for x in package.errata_affected.all()],
            'launchpad_fixed': [x.launchpad for x in package.errata_fixed.all()],
            'git_project': package.git_project,
            'git_commit': package.git_commit,
        })

    return HttpResponse(
        content=json.dumps({
            'status': 'ok',
            'packages': package_out,
        }),
        content_type='application/json',
    )

@csrf_exempt
@token_auth
def api_package_update(request):
    package = request.POST.get('package')
    version = request.POST.get('version')
    release = request.POST.get('release')
    distro = request.POST.get('distro')
    launchpad = request.POST.get('launchpad')
    bundle = request.POST.get('bundle')
    git_project = request.POST.get('git_project')
    git_commit = request.POST.get('git_commit')

    try:
        r = Release.objects.get(name=release)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'release': release,
                'reason': 'Release does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    try:
        d = Distro.objects.get(name=distro, release=r)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'distro': distro,
                'release': r.name,
                'reason': 'Distro does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    p, p_created = Package.objects.get_or_create(
        name=package, version=version, release=r, distro=d)

    if bundle:
        b, b_created = Bundle.objects.get_or_create(
            bundleid=bundle, release=r, distro=d,
        )

        if b_created:
            b.save()

        p.bundle = b
    else:
        b = None

    p.git_commit = git_commit
    p.git_project = git_project

    p.save()

    try:
        e = Erratum.objects.get(launchpad=launchpad)
    except ObjectDoesNotExist:
        return HttpResponse(
            content=json.dumps({
                'status': 'error',
                'launchpad': launchpad,
                'reason': 'Erratum does not exist',
            }),
            content_type='application/json',
            status='404',
            reason='Not Found'
        )

    e.fixed_package.add(p)

    e.save()

    return HttpResponse(
        content=json.dumps({
            'status': 'ok',
            'package': p.name,
            'version': p.version,
            'release': r.name,
            'distro': d.name,
            'launchpad': e.launchpad,
            'git_project': git_project,
            'git_commit': git_commit,
            'bundle': b,
        }),
        content_type='application/json',
    )

def docs(request, page):
    from docutils.core import publish_string

    doc_fp = open('%s/%s.rst' % (settings.DOC_ROOT, page))
    doc = doc_fp.read()
    doc_fp.close()

    return HttpResponse(publish_string(doc, writer_name='html'))
