#!/usr/bin/env python

from docutils.core import publish_string
import os
import re
import sys
import yaml

FILENAME_REGEX = re.compile(r'.*bugs\/(\d+)\/erratum\.yaml$')
PACKAGE_NAME_REGEX = re.compile(r'^([a-z0-9\.\_\-]+)\=([a-z0-9\._\-\~\:\+]+)$')

CURRENT_RELEASE = '6.1'

distros = {
    'centos': 'CentOS',
    'ubuntu': 'Ubuntu'
}

scenario_actions = {
    'service_stop': 'Stop services ',
    'service_start': 'Start services ',
    'service_restart': 'Restart services ',
    'server_down': 'Shutdown ',
    'server_up': 'Start ',
    'server_reboot': 'Reboot ',
    'run_command': 'Run command ',
    'upload_script': 'Upload script ',
    'text': '',
}

scenario_targets = {
    'master': 'Fuel master node',
    'slaves': 'all of OpenStack nodes',
    'controller_role': 'OpenStack controller nodes',
    'compute_role': 'OpenStack compute nodes',
    'cinder_role': 'OpenStack Cinder nodes',
    'ceph-osd_role': 'Ceph-OSD nodes',
    'mongo_role': 'MongoDB nodes',
    'zabbix-server_role': 'Zabbix Server nodes',
    'base-os_role': 'nodes with Base OS role',
}

def compile_scenario(inputdata):
    outhtml = ''
    for distro in inputdata.keys():
        outhtml += '%s: <ul>' % distros[distro]
        for action in inputdata[distro]:
            outhtml += '<li>'
            outhtml += scenario_actions.get(action['type'])

            if action['type'] == 'upload_script':
                outhtml += '<code>%s</code> on ' % action['script']

            if action['type'] == 'run_command':
                outhtml += '<code>%s</code> on ' % action['command']

            if action['type'] == 'text':
                outhtml += publish_string(action['text'], writer_name='html')

            if 'target' in action:
                outhtml += ', '.join([scenario_targets.get(target) for target in action['target']])
            outhtml += '</li>'
        outhtml += '</ul>'

    return outhtml

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "errata_base.settings")

    from errata.models import Erratum, Release, Package, Distro

    filename = sys.argv[1]
    robject = re.match(FILENAME_REGEX, filename)

    if not robject:
        raise Exception('Wrong erratum name')

    lpbug = robject.group(1)

    fp = open(filename, 'r')
    erratumyaml = yaml.load(fp)
    fp.close()

    release, release_created = Release.objects.get_or_create(
        name=CURRENT_RELEASE)
    if release_created:
        release.save()

    erratum, erratum_created = Erratum.objects.get_or_create(
        launchpad=lpbug)

    erratum.title = erratumyaml['title']
    erratum.security = erratumyaml.get('security', False)
    erratum.description = erratumyaml['description']

    if erratum_created:
        erratum.disclosed = False

    if 'cve' in erratumyaml:
        erratum.cve = erratumyaml['cve']

    if 'ossa' in erratumyaml:
        erratum.ossa = erratumyaml['ossa']

    for ptype in ['affected-pkgs']:
        for distroyaml in erratumyaml[ptype]:
            for epkg in erratumyaml[ptype][distroyaml]:
                pobject = re.match(PACKAGE_NAME_REGEX, epkg)
                distro, distro_created = Distro.objects.get_or_create(
                    name=distroyaml, release=release)
                if distro_created:
                    distro.save()
                if pobject:
                    name = pobject.group(1)
                    version = pobject.group(2)
                    package, package_created = Package.objects.get_or_create(
                        name=name, version=version, distro=distro, release=release)

                    if package_created:
                        package.save()

                    if ptype == 'affected-pkgs':
                        erratum.affected_package.add(package)

    for target in erratumyaml['targets']:
        erratum.patch_scenario = compile_scenario(target['patch-scenario'])

    for target in erratumyaml['targets']:
        erratum.verify_scenario = compile_scenario(target['verify-scenario'])

    erratum.save()
